package org.itqop.chatit.handlers;

import net.minecraft.network.chat.ClickEvent;
import net.minecraft.network.chat.HoverEvent;
import net.minecraftforge.eventbus.api.SubscribeEvent;
import net.minecraftforge.event.ServerChatEvent;
import net.minecraftforge.fml.common.Mod;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.server.MinecraftServer;
import net.minecraft.ChatFormatting;
import org.itqop.chatit.utils.Config;
import org.itqop.chatit.utils.PlayerConfigManager;
import org.itqop.chatit.utils.ProfanityChecker;
import org.slf4j.Logger;
import com.mojang.logging.LogUtils;

import java.util.concurrent.CompletableFuture;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import static org.itqop.chatit.utils.Utils.isValidUrl;
import static org.itqop.chatit.utils.Utils.logGlobalMessage;

@Mod.EventBusSubscriber
public class ChatEventHandler {

    private static final Logger LOGGER = LogUtils.getLogger();

    private static final Pattern URL_PATTERN = Pattern.compile(
            "(https?://\\S+)", Pattern.CASE_INSENSITIVE
    );

    @SubscribeEvent
    public static void onServerChat(ServerChatEvent event) {
        MutableComponent errorComponent = createErrorMessage(Config.messageLocal);
        MutableComponent adultComponent = createAdultMessage();

        ServerPlayer sender = event.getPlayer();
        MinecraftServer server = sender.getServer();
        String originalMessage = event.getMessage().getString();

        event.setCanceled(true);

        PlayerConfigManager.ensurePlayerExists(sender);

        boolean isGlobal = originalMessage.startsWith("!");
        String message = isGlobal ? originalMessage.substring(1) : originalMessage;

        MutableComponent chatComponent = createChatMessage(sender, message, isGlobal);

        logGlobalMessage(chatComponent);

        CompletableFuture<Double> future = ProfanityChecker.checkMessageAsync(message);

        future.thenAccept(profanityScore -> {
            if (server != null) {
                server.execute(() -> {
                    double threshold = Config.profanityThreshold;

                    if (profanityScore > threshold && !PlayerConfigManager.hasAdultContentEnabled(sender)) {
                        sender.sendSystemMessage(adultComponent);
                    } else {
                        boolean localMessageSent = false;
                        for (ServerPlayer receiver : server.getPlayerList().getPlayers()) {
                            PlayerConfigManager.ensurePlayerExists(receiver);

                            if (profanityScore > threshold && !PlayerConfigManager.hasAdultContentEnabled(receiver)) {
                                continue;
                            }

                            if (isGlobal) {
                                receiver.sendSystemMessage(chatComponent);
                                continue;
                            }

                            if (receiver.getCommandSenderWorld() != sender.getCommandSenderWorld() ||
                                    receiver.position().distanceTo(sender.position()) > 50 ||
                                    receiver == sender) {
                                continue;
                            }

                            receiver.sendSystemMessage(chatComponent);
                            localMessageSent = true;
                        }

                        if (!isGlobal) {
                            if (!localMessageSent) {
                                sender.sendSystemMessage(errorComponent);
                            } else {
                                sender.sendSystemMessage(chatComponent);
                            }
                        }
                    }
                });
            }
        }).exceptionally(ex -> {
            LOGGER.error(ex.toString());
            if (server != null) {
                server.execute(() -> {
                    MutableComponent errorCustomComponent = createErrorMessage(message);
                    sender.sendSystemMessage(errorCustomComponent);
                });
            }
            return null;
        });
    }

    private static MutableComponent createErrorMessage(String message) {
        MutableComponent openBracket = Component.literal("[");
        MutableComponent errorText = Component.literal("ERROR")
                .withStyle(ChatFormatting.RED);
        MutableComponent closeBracket = Component.literal("] ");

        MutableComponent prefixComponent = openBracket.append(errorText).append(closeBracket);

        Component messageComponent = Component.literal(": " + message).withStyle(ChatFormatting.RED);

        return prefixComponent.append(messageComponent);
    }

    private static MutableComponent createAdultMessage() {
        MutableComponent openBracket = Component.literal("[");
        MutableComponent errorText = Component.literal("ADULT")
                .withStyle(ChatFormatting.RED);
        MutableComponent closeBracket = Component.literal("] ");

        MutableComponent prefixComponent = openBracket.append(errorText).append(closeBracket);

        Component errorMessage = Component.literal(Config.messageAdult).withStyle(ChatFormatting.RED);

        return prefixComponent.append(errorMessage);
    }

    /**
     * Метод для создания сообщения чата с кликабельными ссылками.
     *
     * @param player    Игрок, отправляющий сообщение.
     * @param message   Текст сообщения.
     * @param isGlobal  Флаг, указывающий на глобальное или локальное сообщение.
     * @return Кликабельное сообщение чата.
     */
    private static MutableComponent createChatMessage(ServerPlayer player, String message, boolean isGlobal) {
        MutableComponent openBracket = Component.literal("[");
        MutableComponent letterComponent;
        MutableComponent closeBracket = Component.literal("] ");

        if (isGlobal) {
            letterComponent = Component.literal("G")
                    .withStyle(ChatFormatting.GREEN);
        } else {
            letterComponent = Component.literal("L")
                    .withStyle(ChatFormatting.YELLOW);
        }

        MutableComponent prefixComponent = openBracket.append(letterComponent).append(closeBracket);

        MutableComponent playerName = Component.literal(player.getName().getString())
                .withStyle(style -> style
                        .withClickEvent(new ClickEvent(ClickEvent.Action.SUGGEST_COMMAND, "/msg " + player.getName().getString() + " "))
                        .withHoverEvent(new HoverEvent(HoverEvent.Action.SHOW_TEXT, Component.literal("Нажмите, чтобы ответить")))
                        .withColor(ChatFormatting.WHITE)
                );

        Component separatorComponent = Component.literal(": ");

        MutableComponent messageComponent = Component.empty();
        Matcher matcher = URL_PATTERN.matcher(message);
        int lastEnd = 0;

        while (matcher.find()) {
            if (matcher.start() > lastEnd) {
                String textBeforeUrl = message.substring(lastEnd, matcher.start());
                messageComponent.append(Component.literal(textBeforeUrl));
            }

            String url = matcher.group(1);
            if (isValidUrl(url)) {
                MutableComponent clickableLink = Component.literal(url)
                        .withStyle(style -> style
                                .withClickEvent(new ClickEvent(ClickEvent.Action.OPEN_URL, url))
                                .withHoverEvent(new HoverEvent(HoverEvent.Action.SHOW_TEXT, Component.literal("Нажмите, чтобы открыть ссылку")))
                                .withColor(ChatFormatting.BLUE)
                                .withUnderlined(true)
                        );
                messageComponent.append(clickableLink);
            } else {
                messageComponent.append(Component.literal(url));
            }

            lastEnd = matcher.end();
        }

        if (lastEnd < message.length()) {
            String remainingText = message.substring(lastEnd);
            messageComponent.append(Component.literal(remainingText));
        }

        return prefixComponent.append(playerName).append(separatorComponent).append(messageComponent.withStyle(ChatFormatting.YELLOW));
    }
}
